/*
 * 쐬: 2003/12/08
 *
 */
package com.muimi.zipview;

import java.io.File;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.internal.core.JarEntryFile;
import org.eclipse.jdt.internal.ui.javaeditor.JarEntryEditorInput;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.wizards.datatransfer.ZipFileStructureProvider;

/**
 * @author miya
 *
 */
public class ZipContentProvider extends LabelProvider
implements ITreeContentProvider, IDoubleClickListener {

	private ZipFileStructureProvider zp;
	private String zipFilePath;


//content provider methods
	public Object[] getChildren(Object parent) {
		//System.out.println("getChildren:" +parent.getClass());

		List child = zp.getChildren(parent);
		if(child == null){
			return new Object[0];
		}
		return child.toArray();
	}

	public Object getParent(Object element) {
		return null;
	}

	public boolean hasChildren(Object element) {
		return getChildren(element).length > 0;
	}

	public Object[] getElements(Object input) {

		if(input instanceof IFile){
			IPath path = ((IFile)input).getLocation();
			zipFilePath = path.toFile().getAbsolutePath();
			File file = new File(zipFilePath);
			try {
				ZipFile zipFile = new ZipFile(file);
				zp = new ZipFileStructureProvider(zipFile);
				return getChildren(zp.getRoot());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		return new Object[0];
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}


//label provider methods
	public String getText(Object element) {
		return zp.getLabel(element);
	}

	public Image getImage(Object element) {
		if(zp.isFolder(element)){
			return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_FOLDER);
		}else{
			return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_FILE);			
		}
	}


//double click listener
//TODO EditorI|bvAbv
	public void doubleClick(DoubleClickEvent event) {
		ZipEntry entry = (ZipEntry)((StructuredSelection)event.getSelection()).getFirstElement();
		String fileName = zp.getLabel(entry);
		
		String editorId = null;
		IEditorDescriptor desc = PlatformUI.getWorkbench().getEditorRegistry().getDefaultEditor(fileName);
		if(desc != null){
			editorId = desc.getId();
		}
		if(editorId == null){
			editorId = PlatformUI.getWorkbench().getEditorRegistry().getDefaultEditor().getId();			
		}
		System.out.println(editorId);

		JarEntryFile jarEntryFile = new JarEntryFile(entry.getName(), zipFilePath);
		JarEntryEditorInput editorInput = new JarEntryEditorInput(jarEntryFile);

		try {
			ZipViewPlugin.getPage().openEditor(editorInput, editorId);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}
	
	
//sorter	
	public class ZipEntrySorter extends ViewerSorter{
		public int category(Object element) {
			if(zp.isFolder(element)){
				return 0;
			}else {
				return 1;
			}
		}
	}

	private ZipEntrySorter sorter = new ZipEntrySorter();
	
	public ViewerSorter getSorter(){
		return sorter;
	}
}
